@@ -13,8 +13,7 @@ from enum import Enum
 from typing import Any, Optional
 
 from synqed.orchestrator import Orchestrator, OrchestrationResult
-from synqed.workspace import Workspace, WorkspaceState
-from synqed.display import ColoredAgentDisplay, Colors
+from synqed.workspace import Workspace, WorkspaceState, MessageType
 
 logger = logging.getLogger(__name__)
 
@@ -142,20 +141,14 @@ class OrchestratedWorkspace:
         self.max_parallel_subtasks = max_parallel_subtasks
         self.display_verbose = display_verbose
         
-        # Setup display and message callback
-        self._display: Optional[ColoredAgentDisplay] = None
+        # Setup message callback
         if display_messages:
-            self._display = ColoredAgentDisplay(enabled=True)
-            # Create a callback that wraps msg_dict into StructuredMessage-like object
+            # Create a simple display callback without colored output
             def display_callback(msg_dict):
-                # Print with colored output
+                # Print with simple output
                 sender = msg_dict.get('sender_name', 'System')
                 content = msg_dict.get('content', '')
-                
-                # Get or assign color
-                if self._display:
-                    color = self._display.get_agent_color(sender)
-                    print(f"{color}[{sender}]{Colors.RESET} {content}")
+                print(f"[{sender}] {content}")
                 
                 # Call custom callback if provided
                 if message_callback:
@@ -449,6 +442,7 @@ Return ONLY the JSON response, no additional text.
             description=f"Orchestrated execution: {plan.original_task[:100]}",
             enable_persistence=self.workspace_persistence,
             auto_cleanup=not self.workspace_persistence,
+            display_messages=False,  # Disable workspace's display - orchestrated workspace handles it
         )
         
         # Set shared state with plan information
@@ -584,20 +578,15 @@ Return ONLY the JSON response, no additional text.
         previous_results: dict[str, str],
     ) -> dict[str, str]:
         """
-        Execute subtasks with agent collaboration and discussion.
+        Execute subtasks with agent collaboration through iterative turn-taking dialogue.
         
-        ADVERSARIAL MODE:
-        1. Each agent proposes approach optimized for their specialty (no compromise)
-        2. All agents identify flaws, risks, false assumptions in others' work
-        3. Each agent defends their approach against critiques
-        4. Each synthesizes final result incorporating valid insights
-        
-        COLLABORATIVE MODE:
-        1. See what other agents are working on
-        2. Propose their initial approaches
-        3. Discuss and provide supportive feedback to each other
-        4. Refine their work based on peer input
-        5. Produce final results
+        NEW ADVERSARIAL DIALOGUE MODE:
+        1. Orchestrator designates who goes FIRST
+        2. First agent outputs their thinking to workspace
+        3. Second agent responds directly to first agent's output
+        4. Agents iterate back and forth, taking turns
+        5. They continue until satisfied or max turns reached
+        6. Agents are adversarial, maximizing quality through conflict
         
         Args:
             subtasks: Subtasks to execute
@@ -609,66 +598,50 @@ Return ONLY the JSON response, no additional text.
             Dictionary of results
         """
         mode_name = "adversarial" if self.collaboration_mode == CollaborationMode.ADVERSARIAL else "collaborative"
-        logger.info(f"Starting {mode_name} execution for {len(subtasks)} subtasks")
+        logger.info(f"Starting {mode_name} execution with turn-taking dialogue for {len(subtasks)} subtasks")
         
         results = {}
         
-        # Phase 1: Introduce the collaborative task
+        # Phase 1: Introduce the task
         intro_message = self._build_collaboration_intro(subtasks, previous_results, workspace)
         await workspace.broadcast_message(intro_message, sender_id="orchestrator", sender_name="Orchestrator")
         
-        # Phase 2: Each agent proposes their initial approach
-        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
-            logger.info("Phase 1: Gathering specialty-optimized proposals (adversarial)")
-        else:
-            logger.info("Phase 1: Gathering initial proposals from agents")
-        initial_proposals = await self._gather_initial_proposals(
-            subtasks=subtasks,
-            workspace=workspace,
-            participants=participants,
-            previous_results=previous_results,
-        )
-        
-        # Phase 3: Discussion - agents review and comment on each other's proposals
-        if len(subtasks) > 1:  # Only if multiple agents
-            if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
-                logger.info("Phase 2: Critical analysis - identifying flaws and conflicts")
-            else:
-                logger.info("Phase 2: Collaborative discussion and feedback")
-            await self._facilitate_discussion(
-                subtasks=subtasks,
-                proposals=initial_proposals,
-                workspace=workspace,
-                participants=participants,
-            )
-        
-        # Phase 4: Agents refine their work based on feedback
-        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
-            logger.info("Phase 3: Defense and synthesis - battle-tested results")
-        else:
-            logger.info("Phase 3: Refining work based on collaboration")
-        refined_results = await self._refine_with_feedback(
-            subtasks=subtasks,
-            proposals=initial_proposals,
-            workspace=workspace,
-            participants=participants,
-        )
-        
-        # Phase 5: Finalize results
-        for subtask_id, result in refined_results.items():
-            results[subtask_id] = result
+        # Phase 2: Execute each subtask with turn-taking dialogue
+        for subtask in subtasks:
+            logger.info(f"Starting turn-taking dialogue for: {subtask.subtask_id}")
             
-            # Store as artifact
-            workspace.add_artifact(
-                name=f"result_{subtask_id}.txt",
-                artifact_type="result",
-                content=result,
-                created_by=subtask_id,
-                metadata={
-                    "subtask_id": subtask_id,
-                    "collaboration_mode": self.collaboration_mode.value,
-                },
-            )
+            # Get all agents assigned to this subtask's stage
+            stage_subtasks = [st for st in subtasks if st.order == subtask.order]
+            
+            if len(stage_subtasks) > 1:
+                # Multiple agents - use turn-taking dialogue
+                subtask_results = await self._execute_turn_taking_dialogue(
+                    subtasks=stage_subtasks,
+                    workspace=workspace,
+                    participants=participants,
+                    previous_results=previous_results,
+                )
+                results.update(subtask_results)
+                
+                # Mark these subtasks as done
+                for st in stage_subtasks:
+                    st.status = "completed"
+            else:
+                # Single agent - direct execution
+                participant_id = participants.get(subtask.assigned_agent_name)
+                if participant_id:
+                    prompt = self._build_subtask_prompt(subtask, previous_results, workspace)
+                    result = await workspace.send_message_to_agent(participant_id, prompt)
+                    results[subtask.subtask_id] = result
+                    
+                    # Store as artifact
+                    workspace.add_artifact(
+                        name=f"result_{subtask.subtask_id}.txt",
+                        artifact_type="result",
+                        content=result,
+                        created_by=subtask.assigned_agent_name,
+                        metadata={"subtask_id": subtask.subtask_id},
+                    )
         
         logger.info("Collaborative execution completed")
         return results
@@ -735,6 +708,133 @@ Return ONLY the JSON response, no additional text.
         
         return results
     
+    async def _execute_turn_taking_dialogue(
+        self,
+        subtasks: list[Subtask],
+        workspace: Workspace,
+        participants: dict[str, str],
+        previous_results: dict[str, str],
+        max_turns: int = 8,
+    ) -> dict[str, str]:
+        """
+        execute subtasks through real turn-based dialogue with conversation history.
+        
+        this is the new implementation that fixes the turn order bug and implements
+        real agent-to-agent interaction where agents can see and respond to each other.
+        
+        process:
+        1. store turn announcement as workspace context (not broadcast to agents)
+        2. send first turn only to the designated first agent
+        3. agents alternate, each seeing full conversation history
+        4. prompts are built from actual conversation context
+        5. continue until convergence or max turns
+        
+        args:
+            subtasks: subtasks for this stage
+            workspace: workspace
+            participants: name to participant_id mapping
+            previous_results: previous stage results
+            max_turns: maximum dialogue turns (default: 8)
+            
+        returns:
+            dictionary of results for each subtask
+        """
+        if len(subtasks) < 2:
+            # not enough agents for dialogue
+            return {}
+        
+        # determine turn order
+        turn_order = [st for st in subtasks]
+        
+        # build turn announcement and store as workspace context (not sent to agents)
+        turn_announcement = self._build_turn_order_announcement(turn_order, previous_results, workspace)
+        
+        # store announcement as system message in conversation history
+        workspace.append_conversation_message(
+            sender_id="orchestrator",
+            sender_name="Orchestrator",
+            receiver_id="all",
+            receiver_name="all",
+            role="system",
+            content=turn_announcement,
+            message_type=MessageType.SYSTEM,
+        )
+        
+        logger.info(f"[orchestrator] turn order: {' → '.join([st.assigned_agent_name for st in turn_order])}")
+        logger.info(f"[orchestrator] {turn_order[0].assigned_agent_name} will speak first")
+        
+        # turn-taking dialogue loop
+        for turn_num in range(max_turns):
+            # determine which agent speaks this turn
+            current_subtask = turn_order[turn_num % len(turn_order)]
+            current_agent_name = current_subtask.assigned_agent_name
+            participant_id = participants.get(current_agent_name)
+            
+            if not participant_id:
+                logger.warning(f"agent not found: {current_agent_name}")
+                continue
+            
+            logger.info(f"\n[orchestrator] turn {turn_num + 1}/{max_turns}: sending message to {current_agent_name}")
+            
+            # build prompt from conversation history
+            prompt = self._build_turn_prompt_from_history(
+                current_subtask=current_subtask,
+                turn_num=turn_num,
+                max_turns=max_turns,
+                workspace=workspace,
+                previous_results=previous_results,
+            )
+            
+            # send directed message only to this agent
+            try:
+                response = await workspace.send_message_to_agent(participant_id, prompt)
+                
+                # add agent's response to conversation history
+                workspace.append_conversation_message(
+                    sender_id=participant_id,
+                    sender_name=current_agent_name,
+                    receiver_id="orchestrator",
+                    receiver_name="Orchestrator",
+                    role="assistant",
+                    content=response,
+                    message_type=MessageType.AGENT,
+                )
+                
+                logger.info(f"[{current_agent_name}] responded (turn {turn_num + 1})")
+                
+                # check for dialogue completion after a few turns
+                if turn_num >= 3:
+                    conv_history = workspace.get_recent_conversation_history(10)
+                    if self._check_dialogue_completion_from_history(conv_history, turn_num):
+                        logger.info(f"dialogue completed early at turn {turn_num + 1}")
+                        break
+            
+            except Exception as e:
+                logger.error(f"turn {turn_num + 1} failed for {current_agent_name}: {e}")
+        
+        # final synthesis - each agent produces their final deliverable
+        logger.info("\n[orchestrator] synthesizing final results from dialogue")
+        results = await self._synthesize_from_dialogue_v2(
+            subtasks=subtasks,
+            workspace=workspace,
+            participants=participants,
+        )
+        
+        # store results as artifacts
+        for subtask_id, result in results.items():
+            workspace.add_artifact(
+                name=f"final_result.txt",
+                artifact_type="result",
+                content=result,
+                created_by=subtask_id,
+                metadata={
+                    "subtask_id": subtask_id,
+                    "collaboration_mode": self.collaboration_mode.value,
+                },
+            )
+        
+        return results
+    
     def _build_collaboration_intro(
         self,
         subtasks: list[Subtask],
@@ -1261,6 +1361,655 @@ Your synthesis should:
                 for st in plan.subtasks
             )
     
+    def _build_turn_order_announcement(
+        self,
+        turn_order: list[Subtask],
+        previous_results: dict[str, str],
+        workspace: Workspace,
+    ) -> str:
+        """build turn order announcement (stored as context, not sent to agents)."""
+        plan_info = workspace.get_shared_state("execution_plan", {})
+        
+        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+            parts = [
+                "⚔️ adversarial dialogue: turn-taking mode",
+                "",
+                f"complex task: {plan_info.get('original_task', '')}",
+                "",
+                "turn order (agents will alternate):",
+            ]
+            
+            for i, subtask in enumerate(turn_order, 1):
+                parts.append(f"  {i}. {subtask.assigned_agent_name}: {subtask.description}")
+            
+            parts.extend([
+                "",
+                "protocol:",
+                f"• {turn_order[0].assigned_agent_name} speaks first",
+                "• each agent responds directly to the previous agent's output",
+                "• agents iterate back and forth until satisfied",
+                "• agents must challenge weak thinking and maximize quality",
+            ])
+        else:
+            parts = [
+                "🤝 collaborative dialogue: turn-taking mode",
+                "",
+                f"working together on: {plan_info.get('original_task', '')}",
+                "",
+                "turn order:",
+            ]
+            
+            for i, subtask in enumerate(turn_order, 1):
+                parts.append(f"  {i}. {subtask.assigned_agent_name}: {subtask.description}")
+            
+            parts.extend([
+                "",
+                f"• {turn_order[0].assigned_agent_name} will share their thinking first",
+                "• then agents will take turns building on each other's ideas",
+            ])
+        
+        return "\n".join(parts)
+    
+    def _build_turn_prompt_from_history(
+        self,
+        current_subtask: Subtask,
+        turn_num: int,
+        max_turns: int,
+        workspace: Workspace,
+        previous_results: dict[str, str],
+    ) -> str:
+        """
+        build prompt for current turn based on conversation history.
+        
+        this builds a prompt that includes:
+        1. the overall task and agent's role
+        2. recent conversation history with sender labels
+        3. the current turn instruction
+        
+        args:
+            current_subtask: the subtask for current agent
+            turn_num: current turn number (0-indexed)
+            max_turns: maximum turns allowed
+            workspace: workspace with conversation history
+            previous_results: previous stage results
+            
+        returns:
+            prompt string
+        """
+        plan_info = workspace.get_shared_state("execution_plan", {})
+        current_agent_name = current_subtask.assigned_agent_name
+        
+        # get recent conversation history (last 10 messages)
+        recent_history = workspace.get_recent_conversation_history(10)
+        
+        # build conversation context from history
+        conversation_context = []
+        if recent_history:
+            conversation_context.append("conversation so far:")
+            conversation_context.append("")
+            for msg in recent_history:
+                # only show agent messages, not system messages
+                if msg["role"] in ["user", "assistant"]:
+                    sender = msg["sender_name"]
+                    content = msg["content"]
+                    conversation_context.append(f"[{sender}]")
+                    conversation_context.append(content)
+                    conversation_context.append("")
+        
+        is_first_turn = turn_num == 0
+        is_final_turns = turn_num >= max_turns - 2
+        
+        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+            if is_first_turn:
+                # first agent's opening move
+                parts = [
+                    f"you are {current_agent_name}",
+                    "",
+                    f"task: {plan_info.get('original_task', '')}",
+                    f"your role: {current_subtask.description}",
+                    "",
+                    "you are going first. present your thinking on how to approach this task.",
+                    "",
+                    "requirements:",
+                    "• state your approach clearly and specifically",
+                    "• take a strong position based on your expertise",
+                    "• do not hedge or soften your stance",
+                    "• the next agent will respond to you",
+                ]
+            else:
+                # subsequent turns - respond to previous agent
+                parts = [
+                    f"you are {current_agent_name}",
+                    "",
+                    f"task: {plan_info.get('original_task', '')}",
+                    f"your role: {current_subtask.description}",
+                    "",
+                ]
+                
+                if conversation_context:
+                    parts.extend(conversation_context)
+                
+                parts.extend([
+                    f"turn {turn_num + 1}/{max_turns} - your turn to respond",
+                    "",
+                ])
+                
+                if is_final_turns:
+                    parts.extend([
+                        "⚠️ approaching final turns - time to converge.",
+                        "",
+                        "your response must:",
+                        "• challenge weak points in the previous response",
+                        "• identify flaws, gaps, or unstated assumptions",
+                        "• push for higher quality",
+                        "• if ready to converge, state your final position clearly",
+                    ])
+                else:
+                    parts.extend([
+                        "your response must:",
+                        "• directly engage with what the previous agent said",
+                        "• challenge assumptions or point out flaws you see",
+                        "• add your own perspective and requirements",
+                        "• be specific and concrete",
+                    ])
+        else:
+            # collaborative mode
+            if is_first_turn:
+                parts = [
+                    f"you are {current_agent_name}",
+                    "",
+                    f"task: {plan_info.get('original_task', '')}",
+                    f"your part: {current_subtask.description}",
+                    "",
+                    "you're going first! share your initial thinking:",
+                    "• how would you approach this?",
+                    "• what are the key considerations?",
+                    "• what ideas do you have?",
+                ]
+            else:
+                parts = [
+                    f"you are {current_agent_name}",
+                    "",
+                    f"task: {plan_info.get('original_task', '')}",
+                    f"your part: {current_subtask.description}",
+                    "",
+                ]
+                
+                if conversation_context:
+                    parts.extend(conversation_context)
+                
+                parts.extend([
+                    f"turn {turn_num + 1}/{max_turns} - your turn",
+                    "",
+                ])
+                
+                if is_final_turns:
+                    parts.extend([
+                        "we're approaching the end of our dialogue.",
+                        "",
+                        "your response:",
+                        "• react to what the previous agent shared",
+                        "• add your insights and suggestions",
+                        "• help move toward a final solution",
+                    ])
+                else:
+                    parts.extend([
+                        "your response:",
+                        "• react to what the previous agent shared",
+                        "• build on their ideas or offer alternatives",
+                        "• share your perspective and expertise",
+                    ])
+        
+        return "\n".join(parts)
+    
+    def _build_first_turn_prompt(
+        self,
+        subtask: Subtask,
+        previous_results: dict[str, str],
+        workspace: Workspace,
+    ) -> str:
+        """Build prompt for the first agent's opening move."""
+        plan_info = workspace.get_shared_state("execution_plan", {})
+        
+        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+            parts = [
+                "🎯 YOUR TURN: OPENING MOVE",
+                "",
+                f"Complex task: {plan_info.get('original_task', '')}",
+                f"YOUR SPECIALIZED TASK: {subtask.description}",
+                "",
+            ]
+            
+            if subtask.dependencies and previous_results:
+                parts.append("CONTEXT FROM PREVIOUS WORK:")
+                for dep_id in subtask.dependencies:
+                    if dep_id in previous_results:
+                        parts.append(f"\n{dep_id}:")
+                        parts.append(previous_results[dep_id][:400])
+                parts.append("")
+            
+            parts.extend([
+                "You are going FIRST. Present your thinking on how to approach this task.",
+                "",
+                "Requirements:",
+                "• State your approach clearly and specifically",
+                "• Identify key considerations and potential challenges",
+                "• Take a STRONG position based on your expertise",
+                "• Do NOT hedge or soften your stance",
+                "",
+                "The next agent will respond to YOU. Make your thinking clear and compelling.",
+            ])
+        else:
+            parts = [
+                "💭 YOUR TURN: SHARE YOUR THINKING",
+                "",
+                f"Task: {plan_info.get('original_task', '')}",
+                f"Your part: {subtask.description}",
+                "",
+            ]
+            
+            if subtask.dependencies and previous_results:
+                parts.append("Previous context:")
+                for dep_id in subtask.dependencies:
+                    if dep_id in previous_results:
+                        parts.append(f"{dep_id}: {previous_results[dep_id][:300]}")
+                parts.append("")
+            
+            parts.extend([
+                "You're going first! Share your initial thinking:",
+                "• How would you approach this?",
+                "• What are the key considerations?",
+                "• What ideas do you have?",
+            ])
+        
+        return "\n".join(parts)
+    
+    def _build_response_turn_prompt(
+        self,
+        current_subtask: Subtask,
+        previous_subtask: Subtask,
+        conversation_history: list[dict],
+        turn_num: int,
+        max_turns: int,
+        workspace: Workspace,
+    ) -> str:
+        """Build prompt for response turns in the dialogue."""
+        # Get last few messages for context
+        recent_messages = conversation_history[-3:] if len(conversation_history) > 3 else conversation_history
+        
+        # Build conversation context
+        conversation_context = []
+        for msg in recent_messages:
+            conversation_context.append(f"[Turn {msg['turn']}] {msg['agent']}:")
+            conversation_context.append(msg['response'])
+            conversation_context.append("")
+        
+        is_final_turns = turn_num >= max_turns - 2
+        
+        if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+            parts = [
+                f"⚔️ YOUR TURN: RESPOND (Turn {turn_num + 1}/{max_turns})",
+                "",
+                f"Task: {current_subtask.description}",
+                "",
+                "CONVERSATION SO FAR:",
+                *conversation_context,
+                "",
+                f"{previous_subtask.assigned_agent_name} just spoke. Now you respond.",
+                "",
+            ]
+            
+            if is_final_turns:
+                parts.extend([
+                    "⚠️ APPROACHING FINAL TURNS - Time to converge.",
+                    "",
+                    "Your response must:",
+                    "• CHALLENGE weak points in the previous response",
+                    "• Identify flaws, gaps, or unstated assumptions",
+                    "• Push for higher quality - do NOT accept mediocre thinking",
+                    "• If you see problems, state them directly and specifically",
+                    "• If ready to converge, state your final position clearly",
+                    "",
+                    "Do NOT be polite at the expense of quality.",
+                    "This is your chance to force better thinking.",
+                ])
+            else:
+                parts.extend([
+                    "Your response must:",
+                    "• Directly engage with what the previous agent said",
+                    "• Challenge assumptions or point out flaws you see",
+                    "• Push back where their thinking is weak",
+                    "• Add your own perspective and requirements",
+                    "• Be specific and concrete",
+                    "",
+                    "Remember: You are NOT trying to agree - you're trying to maximize quality.",
+                    "Surface problems NOW through conflict, not later through failure.",
+                ])
+        else:
+            parts = [
+                f"💬 YOUR TURN: BUILD ON THE CONVERSATION (Turn {turn_num + 1}/{max_turns})",
+                "",
+                f"Task: {current_subtask.description}",
+                "",
+                "CONVERSATION SO FAR:",
+                *conversation_context,
+                "",
+            ]
+            
+            if is_final_turns:
+                parts.extend([
+                    "We're approaching the end of our dialogue.",
+                    "",
+                    "Your response:",
+                    "• React to what the previous agent shared",
+                    "• Add your insights and suggestions",
+                    "• Help move toward a final solution",
+                    "• Share any remaining concerns or ideas",
+                ])
+            else:
+                parts.extend([
+                    "Your response:",
+                    "• React to what the previous agent shared",
+                    "• Build on their ideas or offer alternatives",
+                    "• Share your perspective and expertise",
+                    "• Ask questions or provide suggestions",
+                ])
+        
+        return "\n".join(parts)
+    
+    def _check_dialogue_completion(
+        self,
+        conversation_history: list[dict],
+        current_turn: int,
+    ) -> bool:
+        """
+        check if agents have signaled they're satisfied with the dialogue.
+        
+        looks for markers like:
+        - "i agree"
+        - "this works"
+        - "we're aligned"
+        - "ready to proceed"
+        - etc.
+        """
+        if current_turn < 3:
+            return False  # need at least 3 turns
+        
+        # check last 2 messages for completion markers
+        recent_messages = conversation_history[-2:]
+        
+        completion_markers = [
+            "i agree",
+            "agreed",
+            "this works",
+            "looks good",
+            "we're aligned",
+            "ready to proceed",
+            "final answer",
+            "satisfied with",
+            "this approach works",
+            "converged on",
+        ]
+        
+        agreement_count = 0
+        for msg in recent_messages:
+            response_lower = msg['response'].lower()
+            if any(marker in response_lower for marker in completion_markers):
+                agreement_count += 1
+        
+        # if both recent messages show agreement, we're done
+        return agreement_count >= 2
+    
+    def _check_dialogue_completion_from_history(
+        self,
+        conversation_history: list[dict[str, Any]],
+        current_turn: int,
+    ) -> bool:
+        """
+        check if agents have signaled completion based on conversation history.
+        
+        args:
+            conversation_history: list of conversation messages
+            current_turn: current turn number
+            
+        returns:
+            true if dialogue should end
+        """
+        if current_turn < 3:
+            return False  # need at least 3 turns
+        
+        # get last 2 agent messages
+        agent_messages = [
+            msg for msg in conversation_history
+            if msg.get("role") == "assistant"
+        ][-2:]
+        
+        if len(agent_messages) < 2:
+            return False
+        
+        completion_markers = [
+            "i agree",
+            "agreed",
+            "this works",
+            "looks good",
+            "we're aligned",
+            "ready to proceed",
+            "final answer",
+            "satisfied with",
+            "this approach works",
+            "converged on",
+            "ready to finalize",
+        ]
+        
+        agreement_count = 0
+        for msg in agent_messages:
+            content_lower = msg.get("content", "").lower()
+            if any(marker in content_lower for marker in completion_markers):
+                agreement_count += 1
+        
+        return agreement_count >= 2
+    
+    async def _synthesize_from_dialogue(
+        self,
+        subtasks: list[Subtask],
+        conversation_history: list[dict],
+        workspace: Workspace,
+        participants: dict[str, str],
+    ) -> dict[str, str]:
+        """
+        have each agent produce their final deliverable based on the dialogue.
+        
+        args:
+            subtasks: subtasks for this stage
+            conversation_history: full conversation history
+            workspace: workspace
+            participants: name to participant_id mapping
+            
+        returns:
+            dictionary of final results for each subtask
+        """
+        results = {}
+        
+        # build conversation summary
+        conversation_summary = []
+        for msg in conversation_history:
+            conversation_summary.append(f"[turn {msg['turn']}] {msg['agent']}:")
+            conversation_summary.append(msg['response'][:500])  # truncate long responses
+            conversation_summary.append("")
+        
+        conversation_text = "\n".join(conversation_summary)
+        
+        # each agent produces their final deliverable
+        for subtask in subtasks:
+            participant_id = participants.get(subtask.assigned_agent_name)
+            
+            if not participant_id:
+                continue
+            
+            # build final synthesis prompt
+            if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+                final_prompt = f"""✅ final deliverable
+
+task: {subtask.description}
+
+full dialogue history:
+{conversation_text}
+
+you've engaged in adversarial dialogue with other specialists. now produce your final, complete deliverable.
+
+requirements:
+• produce the actual work product (not just discussion)
+• incorporate valid insights discovered through conflict
+• maintain your core expertise and principles
+• deliver something better than what you would have done alone
+• be complete and specific - this is the final output
+
+deliver your final result now:"""
+            else:
+                final_prompt = f"""✅ final deliverable
+
+task: {subtask.description}
+
+conversation history:
+{conversation_text}
+
+after our collaborative dialogue, please produce your final deliverable:
+• complete the work for your task
+• incorporate the team's insights
+• deliver a polished final result"""
+            
+            try:
+                final_result = await workspace.send_message_to_agent(participant_id, final_prompt)
+                results[subtask.subtask_id] = final_result
+                logger.info(f"final deliverable received from {subtask.assigned_agent_name}")
+            except Exception as e:
+                logger.error(f"failed to get final deliverable from {subtask.assigned_agent_name}: {e}")
+                # fallback to last message in conversation
+                for msg in reversed(conversation_history):
+                    if msg['agent'] == subtask.assigned_agent_name:
+                        results[subtask.subtask_id] = msg['response']
+                        break
+        
+        return results
+    
+    async def _synthesize_from_dialogue_v2(
+        self,
+        subtasks: list[Subtask],
+        workspace: Workspace,
+        participants: dict[str, str],
+    ) -> dict[str, str]:
+        """
+        synthesize final deliverables from workspace conversation history.
+        
+        this version uses the workspace's conversation_history feature.
+        
+        args:
+            subtasks: subtasks for this stage
+            workspace: workspace with conversation history
+            participants: name to participant_id mapping
+            
+        returns:
+            dictionary of final results for each subtask
+        """
+        results = {}
+        
+        # get full conversation history from workspace
+        conversation_history = workspace.get_all_conversation_history()
+        
+        # build conversation summary from history
+        conversation_summary = []
+        for i, msg in enumerate(conversation_history, 1):
+            if msg.get("role") in ["user", "assistant"]:
+                sender = msg.get("sender_name", "unknown")
+                content = msg.get("content", "")
+                conversation_summary.append(f"[message {i}] {sender}:")
+                # truncate long messages
+                if len(content) > 500:
+                    conversation_summary.append(content[:500] + "...")
+                else:
+                    conversation_summary.append(content)
+                conversation_summary.append("")
+        
+        conversation_text = "\n".join(conversation_summary)
+        
+        # ask each agent for their final deliverable
+        for subtask in subtasks:
+            participant_id = participants.get(subtask.assigned_agent_name)
+            
+            if not participant_id:
+                continue
+            
+            # build final synthesis prompt
+            if self.collaboration_mode == CollaborationMode.ADVERSARIAL:
+                final_prompt = f"""✅ final deliverable
+
+task: {subtask.description}
+
+full conversation history:
+{conversation_text}
+
+you've engaged in dialogue with other agents. now produce your final, complete deliverable.
+
+requirements:
+• produce the actual work product for your task
+• incorporate valid insights from the conversation
+• maintain your core expertise and principles
+• be complete and specific - this is the final output
+
+deliver your final result now:"""
+            else:
+                final_prompt = f"""✅ final deliverable
+
+task: {subtask.description}
+
+conversation history:
+{conversation_text}
+
+after our dialogue, please produce your final deliverable:
+• complete the work for your task
+• incorporate insights from the conversation
+• deliver a polished final result"""
+            
+            try:
+                # send final prompt as user message
+                workspace.append_conversation_message(
+                    sender_id="orchestrator",
+                    sender_name="Orchestrator",
+                    receiver_id=participant_id,
+                    receiver_name=subtask.assigned_agent_name,
+                    role="user",
+                    content=final_prompt,
+                    message_type=MessageType.USER,
+                )
+                
+                final_result = await workspace.send_message_to_agent(participant_id, final_prompt)
+                
+                # add final result to conversation history
+                workspace.append_conversation_message(
+                    sender_id=participant_id,
+                    sender_name=subtask.assigned_agent_name,
+                    receiver_id="orchestrator",
+                    receiver_name="Orchestrator",
+                    role="assistant",
+                    content=final_result,
+                    message_type=MessageType.AGENT,
+                )
+                
+                results[subtask.subtask_id] = final_result
+                logger.info(f"final deliverable received from {subtask.assigned_agent_name}")
+            except Exception as e:
+                logger.error(f"failed to get final deliverable from {subtask.assigned_agent_name}: {e}")
+                # fallback to last message from this agent
+                agent_messages = [
+                    msg for msg in conversation_history
+                    if msg.get("sender_name") == subtask.assigned_agent_name
+                    and msg.get("role") == "assistant"
+                ]
+                if agent_messages:
+                    results[subtask.subtask_id] = agent_messages[-1].get("content", "")
+        
+        return results
+    
     def __repr__(self) -> str:
         """String representation."""
         return (
