@@ -56,6 +56,9 @@ class WorkspaceMessage:
     sender_name: str
     message_type: MessageType
     content: str
+    receiver_id: str = "all"  # specific agent id or "all" for broadcast
+    receiver_name: str = "all"
+    role: str = "system"  # system, user, assistant, or custom type
     metadata: dict[str, Any] = field(default_factory=dict)
     
     def to_dict(self) -> dict[str, Any]:
@@ -65,6 +68,9 @@ class WorkspaceMessage:
             "timestamp": self.timestamp.isoformat(),
             "sender_id": self.sender_id,
             "sender_name": self.sender_name,
+            "receiver_id": self.receiver_id,
+            "receiver_name": self.receiver_name,
+            "role": self.role,
             "message_type": self.message_type.value,
             "content": self.content,
             "metadata": self.metadata,
@@ -124,10 +130,11 @@ class Workspace:
     
     Example:
         ```python
-        # Create a workspace
+        # Create a workspace with real-time message display
         workspace = Workspace(
             name="Recipe Planning",
-            description="Plan meals and create shopping lists"
+            description="Plan meals and create shopping lists",
+            display_messages=True  # Enable real-time colored message display
         )
         
         # Add agents to the workspace
@@ -137,7 +144,7 @@ class Workspace:
         # Start collaboration
         await workspace.start()
         
-        # Submit a collaborative task
+        # Submit a collaborative task - watch agents communicate in real-time!
         result = await workspace.collaborate(
             "Plan a week of dinners and create a shopping list"
         )
@@ -160,6 +167,7 @@ class Workspace:
         enable_persistence: bool = False,
         max_messages: int = 1000,
         message_retention_hours: int = 24,
+        display_messages: bool = True,
     ):
         """
         Initialize a workspace.
@@ -173,6 +181,7 @@ class Workspace:
             enable_persistence: Whether to persist workspace state to disk
             max_messages: Maximum number of messages to retain
             message_retention_hours: Hours to retain messages
+            display_messages: Enable real-time colored message display (default: True)
         """
         self.workspace_id = workspace_id or str(uuid.uuid4())
         self.name = name
@@ -181,6 +190,7 @@ class Workspace:
         self.enable_persistence = enable_persistence
         self.max_messages = max_messages
         self.message_retention_hours = message_retention_hours
+        self.display_messages = display_messages
         
         # Create workspace directory
         if workspace_dir is None:
@@ -202,6 +212,9 @@ class Workspace:
         self._messages: list[WorkspaceMessage] = []
         self._message_callbacks: list[Callable[[WorkspaceMessage], Coroutine[Any, Any, None]]] = []
         
+        # Conversation history for multi-agent interaction
+        self._conversation_history: list[dict[str, Any]] = []
+        
         # Artifacts
         self._artifacts: dict[str, WorkspaceArtifact] = {}
         
@@ -212,6 +225,10 @@ class Workspace:
         self._message_processor_task: asyncio.Task | None = None
         self._running = False
         
+        # Enable real-time message display if requested
+        if display_messages:
+            self.on_message(self._display_message_callback)
+        
         logger.info(f"Created workspace: {self.name} ({self.workspace_id})")
     
     def add_agent(
@@ -792,6 +809,103 @@ class Workspace:
         """
         self._message_callbacks.append(callback)
     
+    def append_conversation_message(
+        self,
+        sender_id: str,
+        sender_name: str,
+        receiver_id: str,
+        receiver_name: str,
+        role: str,
+        content: str,
+        message_type: MessageType = MessageType.AGENT,
+    ) -> str:
+        """
+        append a message to conversation history for multi-agent interaction.
+        
+        args:
+            sender_id: id of message sender
+            sender_name: name of message sender
+            receiver_id: id of message receiver or "all" for broadcast
+            receiver_name: name of message receiver or "all" for broadcast
+            role: message role (system, user, assistant, etc.)
+            content: message content
+            message_type: type of message
+            
+        returns:
+            message id
+        """
+        message_id = str(uuid.uuid4())
+        timestamp = datetime.now()
+        
+        conv_message = {
+            "id": message_id,
+            "sender": sender_id,
+            "sender_name": sender_name,
+            "receiver": receiver_id,
+            "receiver_name": receiver_name,
+            "role": role,
+            "content": content,
+            "timestamp": timestamp.isoformat(),
+            "message_type": message_type.value,
+        }
+        
+        self._conversation_history.append(conv_message)
+        
+        # also add to regular message log
+        self._add_message(
+            sender_id=sender_id,
+            sender_name=sender_name,
+            message_type=message_type,
+            content=content,
+            receiver_id=receiver_id,
+            receiver_name=receiver_name,
+            role=role,
+            metadata={"message_id": message_id},
+        )
+        
+        return message_id
+    
+    def get_recent_conversation_history(
+        self,
+        n: int,
+        for_agent_id: str | None = None,
+    ) -> list[dict[str, Any]]:
+        """
+        get recent conversation history.
+        
+        args:
+            n: number of recent messages to return
+            for_agent_id: optionally filter for messages relevant to specific agent
+            
+        returns:
+            list of recent conversation messages
+        """
+        if for_agent_id:
+            # filter messages that are either sent by this agent, received by this agent,
+            # or broadcast to all
+            filtered = [
+                msg for msg in self._conversation_history
+                if msg["sender"] == for_agent_id 
+                or msg["receiver"] == for_agent_id 
+                or msg["receiver"] == "all"
+            ]
+            return filtered[-n:] if filtered else []
+        else:
+            return self._conversation_history[-n:] if self._conversation_history else []
+    
+    def get_all_conversation_history(self) -> list[dict[str, Any]]:
+        """
+        get all conversation history.
+        
+        returns:
+            full conversation history
+        """
+        return self._conversation_history.copy()
+    
+    def clear_conversation_history(self) -> None:
+        """clear conversation history."""
+        self._conversation_history.clear()
+    
     async def export_workspace(self, export_path: Path | None = None) -> Path:
         """
         Export the workspace state to a JSON file.
@@ -815,6 +929,7 @@ class Workspace:
             "completed_at": self.completed_at.isoformat() if self.completed_at else None,
             "participants": self.list_participants(),
             "messages": [m.to_dict() for m in self._messages],
+            "conversation_history": self._conversation_history,
             "artifacts": [a.to_dict() for a in self._artifacts.values()],
             "shared_state": self._shared_state,
         }
@@ -872,6 +987,9 @@ class Workspace:
         sender_name: str,
         message_type: MessageType,
         content: str,
+        receiver_id: str = "all",
+        receiver_name: str = "all",
+        role: str = "system",
         metadata: dict[str, Any] | None = None,
     ) -> None:
         """Add a message to the workspace."""
@@ -882,6 +1000,9 @@ class Workspace:
             sender_name=sender_name,
             message_type=message_type,
             content=content,
+            receiver_id=receiver_id,
+            receiver_name=receiver_name,
+            role=role,
             metadata=metadata or {},
         )
         
@@ -926,6 +1047,56 @@ class Workspace:
             # Here you could add message processing logic
             # For now, this is just a placeholder for future enhancements
     
+    async def _display_message_callback(self, message: WorkspaceMessage) -> None:
+        """
+        Built-in callback for displaying messages with colored output.
+        
+        Args:
+            message: The message to display
+        """
+        # Color codes for terminal output
+        RESET = "\033[0m"
+        BOLD = "\033[1m"
+        DIM = "\033[2m"
+        
+        # Colors for different message types and agents
+        COLORS = {
+            "system": "\033[90m",      # Gray
+            "status": "\033[36m",      # Cyan
+            "agent_1": "\033[94m",     # Light Blue
+            "agent_2": "\033[92m",     # Light Green
+            "agent_3": "\033[95m",     # Light Magenta
+            "agent_4": "\033[93m",     # Light Yellow
+            "agent_5": "\033[96m",     # Light Cyan
+        }
+        
+        # Get color based on message type and sender
+        if message.message_type == MessageType.SYSTEM:
+            color = COLORS["system"]
+            prefix = "🔔"
+        elif message.message_type == MessageType.STATUS:
+            color = COLORS["status"]
+            prefix = "⚙️"
+        elif message.message_type == MessageType.AGENT:
+            # Cycle through agent colors based on participant list
+            agent_index = list(self._participants.keys()).index(message.sender_id) if message.sender_id in self._participants else 0
+            color_key = f"agent_{(agent_index % 5) + 1}"
+            color = COLORS.get(color_key, COLORS["agent_1"])
+            prefix = "💬"
+        else:
+            color = RESET
+            prefix = "📝"
+        
+        # Format timestamp
+        timestamp = message.timestamp.strftime("%H:%M:%S")
+        
+        # Display the message with color
+        if message.message_type == MessageType.SYSTEM:
+            print(f"{DIM}{prefix} {message.content}{RESET}")
+        else:
+            print(f"\n{color}{BOLD}{prefix} {message.sender_name}{RESET} {DIM}({timestamp}){RESET}")
+            print(f"{color}   {message.content}{RESET}")
+    
     def __repr__(self) -> str:
         """String representation."""
         return (
